/*
 * Copyright (C) 2019-2021 Eugene 'Vindex' Stulin
 *
 * fileformat is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import std.algorithm : canFind, remove;
import std.getopt;
import std.stdio, std.string;

import amalthea.dataprocessing : getIndex;
import amalthea.fileformats : getFileFormat, contentTypeGuess;
import amalthea.langlocal;
import amalthea.sys : readAppHelpText;


immutable app = "fileformat";
immutable appVersion = import("version").strip;
immutable translations = mixin(import("translations.dtxt"));


int main(string[] args) {
    initLocalization(translations);
    bool fHelp, fVersion, mimeMode;
    args.getopt("help", &fHelp, "version", &fVersion, "m|mime", &mimeMode);
    if (args.length == 1) {
        if (fHelp) {
            writeln(readAppHelpText(app));
        } else if (fVersion) {
            writeln(appVersion);
        } else {
            stderr.writeln("Wrong using of fileformat."._s);
            stderr.writeln("See: fileformat --help"._s);
            return 1;
        }
    } else try {
        auto files = args[1 .. $];
        mimeMode ? showMimeType(files) : showFormat(files);
    } catch(Exception e) {
        stderr.writeln(e.msg._s);
        return 2;
    }
    return 0;
}


/**
 * Prints MIME type and its description for each file.
 * If the array consists of more than one file,
 * the name of each file is also printed.
 */
void showMimeType(string[] files) {
    foreach(i, f; files) {
        if (i > 0) {
            writeln;
        }
        if (!f.exists) {
            stderr.writeln(f ~ ": file not found."._s);
            continue;
        }
        auto mimeAndDescr = contentTypeGuess(f);
        if (files.length > 1) {
            writeln("File: "._s, f);
        }
        writeln("MIME type: "._s, mimeAndDescr[0]);
        writeln("Description: "._s, mimeAndDescr[1]);
    }
}


/**
 * Prints format name, description and group of formats for each file.
 * If the array consists of more than one file,
 * the name of each file is also printed.
 */
void showFormat(string[] files) {
    foreach(i, f; files) {
        if (i > 0) {
            writeln;
        }
        if (!f.exists) {
            stderr.writeln(f ~ ": file not found."._s);
            continue;
        }
        auto format = getFileFormat(f);
        if (format.group.empty) {
            format.group = "(other)"._s;
        }
        if (files.length > 1) {
            writeln("File: "._s, f);
        }
        writeln("Format: "._s, format.format);
        writeln("Description: "._s, format.description);
        writeln("Group: "._s, format.group);
    }
}
